@@ -130,10 +130,11 @@ class MCMC():
             SIGMA_pars = self._PRIORS[par]['pars']
         dist = self._PRIORS[par]['dist']
         if type(dist) == type(gamma):
+            mu = max(mu, .001)
             a = mu**2/sigma**2
+            # a = max(a, .01)
+            if np.isclose(sigma, 0): sigma = np.random.uniform(0, 10)
             scale = sigma**2/mu
-            if a == 0:
-                a = .01
             return (a, 0, scale)
         elif type(dist) == type(invgamma):
             a = mu**2/sigma**2 + 2
@@ -157,7 +158,7 @@ class MCMC():
         elif type(dist) == type(uniform):
             return self._PRIORS[par]['pars']
         elif type(dist) == type(invwishart):
-            return (SIGMA_pars[0], sigma*SIGMA_pars[1])
+            return (SIGMA_pars[0], SIGMA_pars[1])
         else:
             raise TypeError("Distribution doesn't match any of the available: gamma, invgamma, expon, beta, norm, uniform, lognorm")
         
@@ -239,7 +240,7 @@ class MCMC():
                     mp_chains[chain].append(manager.list()) # acceptance
 
                 # Adapt tuning parameters 
-                if ((((iter - 1) % self._TUNING_INTERVAL) == 0) \
+                if ((((iter) % self._TUNING_INTERVAL) == 0) \
                     and iter < self._BURNIN):
                     for chain in range(self._CHAINS):
                         self.__tune(chain)
@@ -259,11 +260,11 @@ class MCMC():
                                 self.__sample_chain(ch, core=ch)
                                 return
                             # If sampling failed at some poing then complete it with prev samples
-                            if len(mp_chains[chain][0]) < self._N_PARS:
+                            if len(mp_chains[ch][0]) < self._N_PARS:
                                 prev_samples = self.samples[ch, self._CURR_ITER[ch] - 1, :]
-                                for n in range(len(mp_chains[chain][0]), self._N_PARS):
-                                    mp_chains[chain][0].append(prev_samples[n])
-                                    mp_chains[chain][1].append(0)
+                                for n in range(len(mp_chains[ch][0]), self._N_PARS):
+                                    mp_chains[ch][0].append(prev_samples[n])
+                                    mp_chains[ch][1].append(0)
                             self.samples[ch, self._CURR_ITER[ch], :] = np.array(mp_chains[ch][0], dtype=object)
                             self.acceptance[ch, self._CURR_ITER[ch], :] = np.array(mp_chains[ch][1])
                             self._CURR_ITER[ch] += 1
@@ -274,11 +275,23 @@ class MCMC():
     
     def __tune(self, chain):
         curr_iter = self._CURR_ITER[chain]
-        if curr_iter == 1:
+        #TODO: if the same value, then sample a new starting point
+        if curr_iter == 0:
             return
         acc_rate_pars = self.acceptance[chain, curr_iter - self._TUNING_INTERVAL:curr_iter, :].mean(axis=0)
         for n_par, acc_rate in enumerate(acc_rate_pars):
             scale = self.tuning_pars[chain, n_par]
+            last_samples = self.samples[chain, curr_iter - self._TUNING_INTERVAL:curr_iter, n_par]
+            # If the sample is the same over time, the replace the last sample
+            # with a random draw from the 50% confidence interval
+            if (n_par + 1) == self._N_PARS:
+                return
+            if len(set(last_samples)) < 3:
+                prior = list(self._PRIORS.values())[n_par]
+                self.samples[chain, curr_iter - 1, n_par] = prior['dist'].ppf(np.random.uniform(.25, .75), *prior['pars'])
+                if np.isclose(scale, 0, 0.0001):
+                    scale = np.random.uniform(0.01*int(1/scale), .1*int(1/scale)) * scale
+
             if acc_rate < 0.001:
                 # reduce by 90 percent
                 self.tuning_pars[chain, n_par] = scale * .1
@@ -313,13 +326,46 @@ class MCMC():
             key: (values - self._RESPONSES_MEAN[key][0]) / self._RESPONSES_MEAN[key][1] 
             for key, values in mu.items()
         }
-        return mu_sd
+        return mu_sd 
+
+
+    def __sample_sigma(self, MUS):
+        '''
+        Sample a random covariance matrix.
+        
+        Arguments:
+        --------------
+            V: np.array
+                covariance matrix (prev)
+            MUS: dict
+                means of the standarized response from the model
+        '''
+        V = np.identity(self._N_RESPONSES)
+        MIN_OBS = self._MIN_OBS
+        MAX_OBS = self._MAX_OBS
+        SIGMA = np.zeros_like(V)
+        for _ in range(MAX_OBS - MIN_OBS + 1):
+            obs = np.array([
+                np.random.choice(value, MIN_OBS, replace=False)
+                for value in self._STD_OBSERVATIONS.values()
+            ]).T
+            mus = np.array([[MUS[key] for key in self._STD_OBSERVATIONS.keys()]])
+            sum_term = np.zeros_like(V)
+            for vec in (obs - mus):
+                sum_term += vec.reshape((self._N_RESPONSES, 1)) @ vec.reshape((1, self._N_RESPONSES))
+            SIGMA += V + sum_term
+
+        SIGMA = SIGMA/(MAX_OBS - MIN_OBS + 1)
+
+        return invwishart.rvs(MIN_OBS+self._N_RESPONSES+2, SIGMA)
 
 
     def __sample_chain(self, chain, parallel=False, mp_chains=None, core=None):
         '''
         Samples a single chain
         '''
+        if self._CURR_ITER[chain] == 0:
+            return
         prev_samples = self.samples[chain, self._CURR_ITER[chain]-1, :]
         accept = 1
         # Sample one parameter at once
@@ -336,19 +382,34 @@ class MCMC():
             # Calculate likelihood of prev samples only if the sample has changed
             if par != 'SIGMA':
                 if accept:
+                    # try:
+                    prev_mu = self.__standarized_response(prev_samples, core=core) 
                     self._likelihood_prev[chain] = self.__likelihood(
-                        self.__standarized_response(prev_samples, core=core), prev_samples[-1]
+                        prev_mu, prev_samples[-1]
                     )
+                    # except OSError:
+                        # warnings.warn(f'DSSAT Simulation failed for the next sample: {par}={prev_sample}')
+                        # self._likelihood_new[chain] = self._likelihood_prev[chain]
+                        # new_samples = 
                 # Calculate likelihood for new sample
                 try:
+                    new_mu = self.__standarized_response(new_samples, core=core)
+                    # Likelihood for new sample is calculated using previous SIGMA.
+                    # That's to avoid new sigma to infere in the new sample's likelihood
                     self._likelihood_new[chain] = self.__likelihood(
-                        self.__standarized_response(new_samples, core=core), new_samples[-1]
+                        new_mu, prev_samples[-1]
                     )
                 # If simulation fails, then continue with the previous samples
                 except OSError:
                     warnings.warn(f'DSSAT Simulation failed for the next sample: {par}={new_sample}')
                     self._likelihood_new[chain] = self._likelihood_prev[chain]
-                    new_samples = prev_samples
+                    new_sample = prev_sample
+                    new_samples[n] = new_sample
+                    
+            else:
+                # Sample sigma
+                MUS = dict(zip(new_mu.keys(), map(np.mean, new_mu.values())))
+                new_sample = self.__sample_sigma(MUS=MUS)
 
             
             if not parallel:
@@ -369,10 +430,12 @@ class MCMC():
 
 
     def __sample_par(self, par, curr_sample, tuning_par):
+        if par == 'SIGMA':
+            return
         pars = self.__moment_match(par, curr_sample, tuning_par)
         sample = self._PRIORS[par]['dist'].rvs(*pars, random_state=int(str(time.time()).split('.')[-1]))
         # Check if sample is within the support
-        if (self._SUPPORT is not None) and (par != 'SIGMA'):
+        if (self._SUPPORT is not None):
             par_support = self._SUPPORT[par]
             if (sample < par_support[0]):
                 return curr_sample
@@ -400,7 +463,7 @@ class MCMC():
             prior.pdf(x_new, *self.__moment_match(par, x, tuning_par))
         R = min(1, np.exp(numerator - denominator) * q_ratio)
         if R > np.random.uniform():
-            if x_new == x:
+            if np.isclose(x, x_new, rtol=.00001):
                 return x, False
             else:
                 return x_new , True
@@ -440,6 +503,8 @@ class MCMC():
                 key: (np.array(values) - np.array(values).mean())/np.array(values).std()
                 for key, values in self._OBSERVATIONS.items()
             }
+            self._MIN_OBS = min(map(len, self._STD_OBSERVATIONS.values()))
+            self._MAX_OBS = max(map(len, self._STD_OBSERVATIONS.values()))
             self._PRIORS['SIGMA'] = {
                 'dist': invwishart,
                 'pars': (
@@ -462,7 +527,7 @@ class MCMC():
         if isinstance(self._PREVIOUS_TRACE, dict):
             prev_samples = self._PREVIOUS_TRACE['samples']
             indexes = list(map(lambda x: hasattr(x, 'shape'), prev_samples[:, :, -1].mean(axis=0)))
-            self._CURR_ITER = np.array([sum(indexes) - 1] * self._CHAINS)
+            self._CURR_ITER = np.array([sum(indexes)] * self._CHAINS)
             
             prev_samples = prev_samples[:, indexes, :]
             self.samples = np.concatenate([prev_samples, self.samples], axis=1)
@@ -479,7 +544,7 @@ class MCMC():
             for n, (_, value) in enumerate(self._PRIORS.items()):
                 for chain in range(self._CHAINS):
                     self.samples[chain, 0, n] = value['dist'].rvs(*value['pars'])
-            self._CURR_ITER = np.array([1] * self._CHAINS)
+            self._CURR_ITER = np.array([0] * self._CHAINS)
 
         self._likelihood_new = np.zeros(shape=(self._CHAINS))
         self._likelihood_prev = np.zeros(shape=(self._CHAINS))
