@@ -2,7 +2,7 @@ from concurrent.futures.process import BrokenProcessPool
 from distutils.log import warn
 import enum
 from turtle import position
-from attr import has
+# from attr import has
 import pandas as pd
 from numpy import cumsum, isin
 from .exceptions import DSSATInputError
@@ -16,7 +16,7 @@ class outFile(list):
     def __str__(self) -> str:
         return ''.join(self)
 
-
+# TODO: Allow the treatment to be pased as integer
 class DSSATOutput(dict):
     '''
     A dict-like class to handle DSSAT Output Files. 
@@ -54,7 +54,16 @@ class DSSATOutput(dict):
     def getOutput(self, outName, treat):
         '''
         Get the output for the specified OutName and treatment
+        ...
+        Attributes
+        ------------------
+        outName: str
+            Name of the output file. Don't include the .OUT extension
+
+        treat: str
+            Name of the treatment to retrieve
         '''
+        if not isinstance(treat, str): treat = str(treat)
         return self.get(treat).get(outName)
         
 
@@ -149,6 +158,11 @@ class CropParser():
             for n, line in enumerate(self._FILELINES)
             if line.strip()[:2] == '!*'
         }
+        if len(self._SEC_START_LINES) == 0:
+            raise DSSATInputError(
+                f'No header sections were found on {self.CROPFILE}' + 
+                'This file probably not valid for CROPGRO model'
+            )
         self.RAISE_MISS = raise_missing_pars
         self.parameters = {}
         self._parsloc = {}
@@ -163,7 +177,10 @@ class CropParser():
         self.__read_root()
         self.__read_canopy()
         self.__read_ET()
-        self.__read_dormancy()
+        try:
+            self.__read_dormancy()
+        except KeyError:
+            warn(f'\033[93mWarn: dormancy section was not found at {self.CROPFILE}')
         self.__read_cul()
         self.__read_eco()
         return
@@ -207,7 +224,7 @@ class CropParser():
         curr_line: int
             Current line on the crop file being read
         '''
-        if not all([name in line for name in names]):
+        if not any([name in line for name in names]):
             if self.RAISE_MISS:
                 raise DSSATInputError(
                     f'One or some of {", ".join(names)} parameters were not' +
